/*
  Complete the program to do as described by the usage statement.

  Run your program with the following settings for the arguments, and 
  for each copy/paste the program output and also an explanation of anything
  interesting that could be said about the results.  

  ./memory_demo.o 10 31

  ./memory_demo.o 1000000000 4

  ./memory_demo.o 1000000000 8

  ./memory_demo.o 1000000000 9

  ./memory_demo.o 1000000000 16

  ./memory_demo.o 1000000000 17

  Remember that the numbers given as command-line arguments are in decimal, 
  but your program needs to look at the bits of the second argument to 
  determine the settings to use.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

int check_params(int argc, char *argv[]) {
  if (argc < 3 || atoi(argv[1]) <= 0) return -1;

  char *flags = argv[2];
  if (strlen(flags) <= 0) return -1;

  for(int i=0; i < flags[i] != '\0'; i++)
    if (!isdigit(flags[i])) return -1;

  return atoi(flags);
}

int main(int argc, char *argv[]) {
  time_t begin = time(NULL);
  
  int flags = check_params(argc, argv);
  if (flags <= 0) {
    printf("Usage: ./memory_demo.o num_bytes flags\n"
	   " The program allocates a specified number of bytes (chars) of memory\n"
	   "  If num_bytes is <= 0 the program will abort.\n"
	   "  The program does /not/ check for a maximum allowed value - "
	   "   if it is too large, the program will still\n"
	   "  attempt to allocate space and will then crash.\n"
	   "\n"
	   " flags is an integer bit mask, with each bit having the following meaning\n"
	   "  flags &  0x1 - loop through and set each byte to 65\n"
	   "  flags &  0x2 - loop through and print each byte\n"
	   "  flags &  0x4 - stack test - allocate array as local variable\n"
	   "  flags &  0x8 - heap test - allocate using malloc\n"
	   "  flags & 0x10 - globals test - use memory that is declared as a global\n"
	   "                 your program should declare a global variable that is \n"
	   "                 100 million chars that will be used for this test\n"
	   "\n"
	   " Note that if more than one of the test flags is set then the program will\n"
	   "  perform more than one test.\n"
	   "\n"
	   " The program will print the the beginning and ending address of each array\n"
	   "  used in the testing, and also the total # of seconds elapsed for the test.\n"
	   );
    exit(0);
  }

  return 0;
}